import { Injectable } from "@angular/core";
import { Storage } from "@ionic/storage";
import * as openpgp from 'openpgp';
import { P2pDatabaseGunProvider } from "../../providers/p2p-database-gun/p2p-database-gun";

@Injectable()
export class PgpKeyServerProvider {

  hkp = new openpgp.HKP('https://sks-keyservers.net/');
  pk: any[] = [];
  passphrase = "passphrase";

  constructor(private storage: Storage) {}

  public async generateKey(passphrase, email) {
    let options = {
      userIds: [{ email: email }], // multiple user IDs
      curve: "ed25519", // ECC curve name
      passphrase: this.passphrase // protects the private key
    };

    let a = await openpgp.generateKey(options);
    return a;
  }

  public async publishPubKey(pubkey) {
    this.hkp.upload(pubkey).then(function(result) {
      console.log('public key successfully uploaded',result);
    });
  }

  public async lookupKeys(email: string) {
    // console.log('looking up keys for',email);
    var options = {
      query: email
    };
    try{
      let armoredPubkey = await this.hkp.lookup(options);
      let pubkey = (await openpgp.key.readArmored(armoredPubkey)).keys[0];
      this.pk.push(pubkey);
      return pubkey;
    }
    catch(err){
      console.log("Error: key not found",err);
      return "Key not found";
    }
    
  }

  /**
   * Encrypt text with RSA
   * @param plainText plain text
   * @param privateKey private key
   */
  public async encrypt(plainText) {
    if(!this.pk){ console.log("this.pk is empty"); return;}

    const options = {
      message: openpgp.message.fromText(plainText), // input as Message object
      publicKeys: await Promise.all(this.pk), // for encryption
      // privateKey s: [privKeyObj] // for signing (optional)
    }
    const ciphertext = await openpgp.encrypt(options);
    return ciphertext.data;
  }

  public async decrypt(encrypted: string,a) {
    const privKeyObj = (await openpgp.key.readArmored(a)).keys[0];
    await privKeyObj.decrypt(this.passphrase);
    const options2 = {
      message: await openpgp.message.readArmored(encrypted), // parse armored message
      privateKeys: [privKeyObj] // for decryption
    }
    try{
       let plaintext = await openpgp.decrypt(options2);
      return plaintext.data // 'Hello, World!'
    }
     catch(err){
      console.log('Error thrown:',err);
    }
   return null;
  }

  public async revokeKey() {
    //using revocation certificate
    let pubkey = await this.storage.get("publicKey");
    let atest = (await openpgp.key.readArmored(pubkey)).keys[0];
    let revocatnCert = this.storage.get("revocationCert");
    try {
      var options = {
        key: atest,
        revocationCertificate: revocatnCert
      };

      openpgp.revokeKey(options).then(function(key) {
        console.log("public key revoked", key);
      });

    } catch (e) {
      console.log('revoke failed1', e);
    }

  }

}
